# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .. import PROTOCOL_ID
from pyscada.models import DeviceProtocol
from pyscada.device import GenericHandlerDevice

try: 
    import serial
    driver_ok = True
except ImportError:
    serial = None
    driver_ok = False

from time import time

import logging

logger = logging.getLogger(__name__)


class GenericDevice(GenericHandlerDevice):
    def __init__(self, pyscada_device, variables):
        super().__init__(pyscada_device, variables)
        self._protocol = PROTOCOL_ID
        self.driver_ok = driver_ok

    def connect(self):
        """
        establish a connection to the Instrument
        """
        super().connect()
        result = True

        try:
            self.inst = serial.Serial(port=self._device.serialdevice.port,
                                      baudrate=self._device.serialdevice.baudrate,
                                      bytesize=self._device.serialdevice.bytesize,
                                      parity=self._device.serialdevice.parity,
                                      stopbits=self._device.serialdevice.stopbits,
                                      timeout=self._device.serialdevice.timeout,
                                      write_timeout=self._device.serialdevice.timeout)
        except serial.serialutil.SerialException as e:
            logger.debug(e)
            result = False

        self.accessibility()
        return result

    def disconnect(self):
        if self.inst is not None:
            self.inst.close()
            self.inst = None
            return True
        return False



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _

from . import __app_name__


class PyScadaSerialConfig(AppConfig):
    name = 'pyscada.' + __app_name__.lower()
    verbose_name = _("PyScada " + __app_name__)
    path = os.path.dirname(os.path.realpath(__file__))
    default_auto_field = 'django.db.models.AutoField'

    def ready(self):
        __import__('pyscada.' + __app_name__.lower() + '.signals')



# Generated by Django 2.2.8 on 2021-06-01 12:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('serial', '0010_devicehandler_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='serialdevice',
            name='instrument',
        ),
    ]



# Generated by Django 2.2.8 on 2020-09-03 14:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('pyscada', '0059_auto_20200211_1049'),
    ]

    operations = [
        migrations.CreateModel(
            name='ExtendedSerialDevice',
            fields=[
            ],
            options={
                'verbose_name': 'Serial Device',
                'verbose_name_plural': 'Serial Devices',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('pyscada.device',),
        ),
        migrations.CreateModel(
            name='ExtendedSerialVariable',
            fields=[
            ],
            options={
                'verbose_name': 'Serial Variable',
                'verbose_name_plural': 'Serial Variables',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('pyscada.variable',),
        ),
        migrations.CreateModel(
            name='SerialVariable',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('serial_variable', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='pyscada.Variable')),
            ],
        ),
        migrations.CreateModel(
            name='SerialDevice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('serial_device', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='pyscada.Device')),
            ],
        ),
    ]



# Generated by Django 2.2.8 on 2020-09-03 15:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('serial', '0003_auto_20200903_1507'),
    ]

    operations = [
        migrations.AlterField(
            model_name='serialdevice',
            name='baudrate',
            field=models.PositiveIntegerField(default=0, help_text='0 use default'),
        ),
    ]



# Generated by Django 2.2.8 on 2021-06-01 09:38

from django.db import migrations, models
import logging

logger = logging.getLogger(__name__)


def move_serial_device_handlers(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    SerialDevice = apps.get_model("serial", "SerialDevice")
    SerialHandler = apps.get_model("serial", "SerialDeviceHandler")
    Handler = apps.get_model("pyscada", "DeviceHandler")
    items = []
    count = 0
    for item in SerialHandler.objects.using(schema_editor.connection.alias).all():
        items.append(Handler(name=item.name,
                             handler_class=item.handler_class,
                             handler_path=item.handler_path,
                             ))
        count += 1

    Handler.objects.bulk_create(items)

    devices = []
    for item in SerialHandler.objects.using(schema_editor.connection.alias).all():
        for device in SerialDevice.objects.using(schema_editor.connection.alias).all():
            if device.instrument == item:
                device.instrument_handler = Handler.objects.filter(name=item.name,
                                                                   handler_class=item.handler_class,
                                                                   handler_path=item.handler_path,).first()
                devices.append(device)

        # item.delete()
    SerialDevice.objects.bulk_update(devices, ['instrument_handler'])

    logger.info('moved %d SerialHandler\n' % count)


class Migration(migrations.Migration):

    dependencies = [
        ('pyscada', '0079_devicehandler'),
        ('serial', '0009_serialdevice_instrument_handler')
    ]

    operations = [
        migrations.RunPython(move_serial_device_handlers, reverse_code=migrations.RunPython.noop),
    ]



# Generated by Django 2.2.8 on 2021-06-01 12:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('serial', '0011_remove_serialdevice_instrument'),
    ]

    operations = [
        migrations.DeleteModel(
            name='SerialDeviceHandler',
        ),
    ]



# Generated by Django 3.2 on 2023-02-10 10:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('serial', '0013_devicehandler_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='serialdevice',
            name='instrument_handler',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from pyscada.models import Device, DeviceHandler
from pyscada.models import Variable
from . import PROTOCOL_ID

import serial

from django.db import models
import logging

logger = logging.getLogger(__name__)


class SerialDevice(models.Model):
    serial_device = models.OneToOneField(Device, null=True, blank=True, on_delete=models.CASCADE)
    protocol_choices = ((0, 'serial AT'),)
    protocol = models.PositiveSmallIntegerField(default=0, choices=protocol_choices)
    port = models.CharField(default='/dev/ttyAMA0',
                            max_length=400,
                            help_text="enter serial port (/dev/pts/13))")
    timeout = models.PositiveSmallIntegerField(default=0, help_text="0 use default, else value in seconds")
    stopbits_choices = ((1, 'one stopbit'),
                        (1.5, 'one point five stopbit'),
                        (2, '2 stopbits'),)
    stopbits = models.FloatField(default=serial.STOPBITS_ONE, choices=stopbits_choices)
    bytesize_choices = ((5, 'FIVEBITS'), (6, 'SIXBITS'), (7, 'SEVENBITS'), (8, 'EIGHTBITS'),)
    bytesize = models.PositiveSmallIntegerField(default=serial.EIGHTBITS, choices=bytesize_choices)
    parity_choices = ((serial.PARITY_NONE, 'NONE'), (serial.PARITY_EVEN, 'EVEN'), (serial.PARITY_ODD, 'ODD'),
                      (serial.PARITY_MARK, 'MARK'), (serial.PARITY_SPACE, 'SPACE'),)
    parity = models.CharField(default=serial.PARITY_NONE, max_length=254, choices=parity_choices)
    baudrate = models.PositiveIntegerField(default=9600, help_text="0 use default")

    protocol_id = PROTOCOL_ID

    def parent_device(self):
        try:
            return self.serial_device
        except:
            return None

    def __str__(self):
        return self.serial_device.short_name


class SerialVariable(models.Model):
    serial_variable = models.OneToOneField(Variable, null=True, blank=True, on_delete=models.CASCADE)
    device_property = models.CharField(default='present_value', max_length=255,
                                       help_text='name of the Property the variable be assigned to')

    protocol_id = PROTOCOL_ID

    def __str__(self):
        return self.id.__str__() + "-" + self.serial_variable.name


class ExtendedSerialDevice(Device):
    class Meta:
        proxy = True
        verbose_name = 'Serial Device'
        verbose_name_plural = 'Serial Devices'


class ExtendedSerialVariable(Variable):
    class Meta:
        proxy = True
        verbose_name = 'Serial Variable'
        verbose_name_plural = 'Serial Variables'



# Generated by Django 2.2.8 on 2020-09-04 08:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('serial', '0005_serialdevice_instrument'),
    ]

    operations = [
        migrations.AlterField(
            model_name='serialdevice',
            name='baudrate',
            field=models.PositiveIntegerField(default=9600, help_text='0 use default'),
        ),
        migrations.AlterField(
            model_name='serialdevice',
            name='bytesize',
            field=models.CharField(choices=[(5, 'FIVEBITS'), (6, 'SIXBITS'), (7, 'SEVENBITS'), (8, 'EIGHTBITS')], default=8, max_length=254),
        ),
        migrations.AlterField(
            model_name='serialdevice',
            name='parity',
            field=models.CharField(choices=[('N', 'NONE'), ('E', 'EVEN'), ('O', 'ODD'), ('M', 'MARK'), ('S', 'SPACE')], default='N', max_length=254),
        ),
        migrations.AlterField(
            model_name='serialdevice',
            name='port',
            field=models.CharField(default='/dev/ttyAMA0', help_text='enter serial port (/dev/pts/13))', max_length=400),
        ),
        migrations.AlterField(
            model_name='serialdevice',
            name='stopbits',
            field=models.CharField(choices=[(1, 'one stopbit'), (1.5, 'one point five stopbit'), (2, '2 stopbits')], default=1, max_length=254),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from pyscada.device import GenericDevice
from .devices import GenericDevice as GenericHandlerDevice

from time import time, sleep

import sys

import logging
logger = logging.getLogger(__name__)

try:
    import serial
    driver_ok = True
except ImportError:
    logger.error('Cannot import serial')
    driver_ok = False


class Device(GenericDevice):
    """
    Serial device
    """

    def __init__(self, device):
        self.driver_ok = driver_ok
        self.handler_class = GenericHandlerDevice
        super().__init__(device)

        for var in self.device.variable_set.filter(active=1):
            if not hasattr(var, 'serialvariable'):
                continue
            self.variables[var.pk] = var

    def write_data(self, variable_id, value, task):
        """
        write value to a Serial Device
        """

        output = []
        if not self.driver_ok:
            logger.error("Cannot import serial")
            return output

        self._h.connect()

        if self._h.inst is None:
            return output

        output = super().write_data(variable_id, value, task)

        self._h.disconnect()

        return output



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from pyscada.serial.devices import GenericDevice
from serial.serialutil import SerialException

import logging

logger = logging.getLogger(__name__)


class Handler(GenericDevice):
    """
    AirLink GX450 and other Devices with the same command set
    """

    def read_data(self, variable_instance):
        """
        read values from the device
        """
        if self.inst is None:
            return
        if variable_instance.serialvariable.device_property.upper() == 'SMSM2M':
            return None
        else:
            self.inst.write(str("AT" + variable_instance.serialvariable.device_property.upper() + "\r\n").encode())
            try:
                value = self.parse_value(str(self.inst.readall().decode()))
            except SerialException as e:
                logger.info("Serial Exception reading %s from %s : %s"
                            % (variable_instance.serialvariable.device_property.upper(),
                               variable_instance.device,
                               str(e)))
                value = None
            return value

    def write_data(self, variable_id, value, task):
        """
        write values to the device
        """
        variable = self._variables[variable_id]
        if variable.serialvariable.device_property.upper() == 'SMSM2M':
            return self.parse_value(str(self.inst.readall().decode()))
        try:
            value = self.parse_value(str(self.inst.readall().decode()))
        except SerialException as e:
            logger.info("Serial Exception reading %s from %s : %s"
                        % (variable.serialvariable.device_property.upper(),
                           variable.device,
                           str(e)))
            value = None
        return value

    def parse_value(self, value):
        """
        takes a string in the AirLink GX450 format and returns a float value or None if not parseable
        """
        try:
            return float(value.split()[1])
        except:
            return None



# Generated by Django 2.2.8 on 2021-06-01 12:11

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('pyscada', '0079_devicehandler'),
        ('serial', '0008_auto_20200904_0837'),
    ]

    operations = [
        migrations.AddField(
            model_name='serialdevice',
            name='instrument_handler',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='pyscada.DeviceHandler'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from pyscada.models import Device, Variable
from pyscada.serial.models import SerialDevice, SerialVariable, ExtendedSerialVariable, \
    ExtendedSerialDevice

from django.dispatch import receiver
from django.db.models.signals import post_save

import logging

logger = logging.getLogger(__name__)


@receiver(post_save, sender=SerialDevice)
@receiver(post_save, sender=SerialVariable)
@receiver(post_save, sender=ExtendedSerialVariable)
@receiver(post_save, sender=ExtendedSerialDevice)
def _reinit_daq_daemons(sender, instance, **kwargs):
    """
    update the daq daemon configuration when changes be applied in the models
    """
    if type(instance) is SerialDevice:
        post_save.send_robust(sender=Device, instance=instance.serial_device)
    elif type(instance) is SerialVariable:
        post_save.send_robust(sender=Variable, instance=instance.serial_variable)
    elif type(instance) is ExtendedSerialVariable:
        post_save.send_robust(sender=Variable, instance=Variable.objects.get(pk=instance.pk))
    elif type(instance) is ExtendedSerialDevice:
        post_save.send_robust(sender=Device, instance=Device.objects.get(pk=instance.pk))



# Generated by Django 2.2.8 on 2021-06-01 09:38

from django.db import migrations, models
import logging

logger = logging.getLogger(__name__)


def move_device_handlers(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    Device = apps.get_model("serial", "SerialDevice")
    count = 0
    for item in Device.objects.using(schema_editor.connection.alias).all():
        item.serial_device.instrument_handler = item.instrument_handler
        item.serial_device.save()
        count += 1

    logger.info('moved %d Serial Handler\n' % count)


class Migration(migrations.Migration):

    dependencies = [
        ('serial', '0012_auto_20210601_1244'),
        ('pyscada', '0100_device_instrument_handler')
    ]

    operations = [
        migrations.RunPython(move_device_handlers, reverse_code=migrations.RunPython.noop),
    ]



# Generated by Django 2.2.8 on 2020-09-04 08:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('serial', '0007_auto_20200904_0832'),
    ]

    operations = [
        migrations.AlterField(
            model_name='serialdevice',
            name='bytesize',
            field=models.PositiveSmallIntegerField(choices=[(5, 'FIVEBITS'), (6, 'SIXBITS'), (7, 'SEVENBITS'), (8, 'EIGHTBITS')], default=8),
        ),
        migrations.AlterField(
            model_name='serialdevice',
            name='stopbits',
            field=models.FloatField(choices=[(1, 'one stopbit'), (1.5, 'one point five stopbit'), (2, '2 stopbits')], default=1),
        ),
    ]



# Generated by Django 2.2.8 on 2020-09-03 15:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('serial', '0002_add_device_protocol'),
    ]

    operations = [
        migrations.CreateModel(
            name='SerialDeviceHandler',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='', max_length=255)),
                ('handler_class', models.CharField(default='pyscada.serial.devices.AirLinkGX450', help_text='a Base class to extend can be found at pyscada.serial.devices.GenericDevice', max_length=255)),
                ('handler_path', models.CharField(blank=True, default=None, max_length=255, null=True)),
            ],
        ),
        migrations.AddField(
            model_name='serialdevice',
            name='baudrate',
            field=models.PositiveSmallIntegerField(default=0, help_text='0 use default'),
        ),
        migrations.AddField(
            model_name='serialdevice',
            name='bytesize',
            field=models.PositiveSmallIntegerField(choices=[(0, 'default'), (5, 'FIVEBITS'), (6, 'SIXBITS'), (7, 'SEVENBITS'), (8, 'EIGHTBITS')], default=0),
        ),
        migrations.AddField(
            model_name='serialdevice',
            name='parity',
            field=models.PositiveSmallIntegerField(choices=[(0, 'default'), (1, 'NONE'), (2, 'EVEN'), (3, 'ODD')], default=0),
        ),
        migrations.AddField(
            model_name='serialdevice',
            name='port',
            field=models.CharField(default='502', help_text='enter serial port (/dev/pts/13))', max_length=400),
        ),
        migrations.AddField(
            model_name='serialdevice',
            name='protocol',
            field=models.PositiveSmallIntegerField(choices=[(0, 'serial AT')], default=0),
        ),
        migrations.AddField(
            model_name='serialdevice',
            name='stopbits',
            field=models.PositiveSmallIntegerField(choices=[(0, 'default'), (1, 'one stopbit'), (2, '2 stopbits')], default=0),
        ),
        migrations.AddField(
            model_name='serialdevice',
            name='timeout',
            field=models.PositiveSmallIntegerField(default=0, help_text='0 use default, else value in seconds'),
        ),
        migrations.AddField(
            model_name='serialvariable',
            name='device_property',
            field=models.CharField(default='present_value', help_text='name of the Property the variable be assigned to', max_length=255),
        ),
    ]



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from pyscada.utils.scheduler import SingleDeviceDAQProcessWorker
from pyscada.serial import PROTOCOL_ID

import logging


logger = logging.getLogger(__name__)


class Process(SingleDeviceDAQProcessWorker):
    device_filter = dict(serialdevice__isnull=False, protocol_id=PROTOCOL_ID)
    bp_label = 'pyscada.serial-%s'

    def __init__(self, dt=5, **kwargs):
        super(SingleDeviceDAQProcessWorker, self).__init__(dt=dt, **kwargs)



# Generated by Django 2.2.8 on 2020-09-04 08:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('serial', '0006_auto_20200904_0824'),
    ]

    operations = [
        migrations.AlterField(
            model_name='serialdevice',
            name='bytesize',
            field=models.CharField(choices=[('5', 'FIVEBITS'), ('6', 'SIXBITS'), ('7', 'SEVENBITS'), ('8', 'EIGHTBITS')], default=8, max_length=254),
        ),
        migrations.AlterField(
            model_name='serialdevice',
            name='stopbits',
            field=models.CharField(choices=[('1', 'one stopbit'), ('1.5', 'one point five stopbit'), ('2', '2 stopbits')], default=1, max_length=254),
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import os
from pyscada import serial as pyscada_app


CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Environment :: Console',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
    'Operating System :: POSIX',
    'Operating System :: MacOS :: MacOS X',
    'Programming Language :: Python',
    'Programming Language :: JavaScript',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    'Topic :: Scientific/Engineering :: Visualization'
]
setup(
    author=pyscada_app.__author__,
    author_email=pyscada_app.__email__,
    name='pyscada-' + pyscada_app.__app_name__.lower(),
    version=pyscada_app.__version__,
    description=pyscada_app.__description__,
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    url='http://www.github.com/pyscada/PyScada-' + pyscada_app.__app_name__,
    license='AGPLv3',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    install_requires=[
        'pyscada>=0.8.0',
        'pyserial',
    ],
    packages=find_packages(exclude=["project", "project.*"]),
    include_package_data=True,
    zip_safe=False,
    test_suite='runtests.main'
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

__version__ = "0.8.0"
__author__ = "Camille Lavayssiere"
__email__ = "team@pyscada.org"
__description__ = "Serial extension for PyScada a Python and Django based Open Source SCADA System"
__app_name__ = "Serial"

PROTOCOL_ID = 93

parent_process_list = [{'pk': PROTOCOL_ID,
                        'label': 'pyscada.' + __app_name__.lower(),
                        'process_class': 'pyscada.' + __app_name__.lower() + '.worker.Process',
                        'process_class_kwargs': '{"dt_set":30}',
                        'enabled': True}]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2017-02-24 12:49
from __future__ import unicode_literals
from pyscada.serial import PROTOCOL_ID

from django.db import migrations


def forwards_func(apps, schema_editor):
    # We get the model from the versioned app registry;
    # if we directly import it, it'll be the wrong version
    DeviceProtocol = apps.get_model("pyscada", "DeviceProtocol")
    db_alias = schema_editor.connection.alias
    if not DeviceProtocol.objects.using(db_alias).filter(pk=PROTOCOL_ID):
        DeviceProtocol.objects.using(db_alias).bulk_create([
            DeviceProtocol(pk=PROTOCOL_ID,
                           protocol='serial',
                           description='Serial Interface',
                           app_name='pyscada.serial',
                           device_class='pyscada.serial.device',
                           daq_daemon=True,
                           single_thread=True),
        ])


def reverse_func(apps, schema_editor):
    # forwards_func() creates two Country instances,
    # so reverse_func() should delete them.
    DeviceProtocol = apps.get_model("pyscada", "DeviceProtocol")
    db_alias = schema_editor.connection.alias
    DeviceProtocol.objects.using(db_alias).filter(protocol="serial").delete()


class Migration(migrations.Migration):
    dependencies = [
        ('serial', '0001_initial'),
        ('pyscada', '0059_auto_20200211_1049'),
    ]

    operations = [
        migrations.RunPython(forwards_func, reverse_func),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from pyscada.serial import PROTOCOL_ID
from pyscada.serial.models import SerialDevice, ExtendedSerialDevice
from pyscada.serial.models import SerialVariable, ExtendedSerialVariable
from pyscada.admin import DeviceAdmin
from pyscada.admin import VariableAdmin
from pyscada.admin import admin_site
from pyscada.models import Device, DeviceProtocol
from django.contrib import admin
import logging

logger = logging.getLogger(__name__)


class SerialDeviceAdminInline(admin.StackedInline):
    model = SerialDevice


class SerialDeviceAdmin(DeviceAdmin):
    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == 'protocol':
            kwargs['queryset'] = DeviceProtocol.objects.filter(pk=PROTOCOL_ID)
            db_field.default = PROTOCOL_ID
        return super(SerialDeviceAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)

    def get_queryset(self, request):
        """Limit Pages to those that belong to the request's user."""
        qs = super(SerialDeviceAdmin, self).get_queryset(request)
        return qs.filter(protocol_id=PROTOCOL_ID)

    inlines = [
        SerialDeviceAdminInline
    ]


class SerialVariableAdminInline(admin.StackedInline):
    model = SerialVariable


class SerialVariableAdmin(VariableAdmin):
    list_display = ('id', 'name', 'description', 'unit', 'device_name', 'value_class', 'active', 'writeable')
    list_editable = ('active', 'writeable',)
    list_display_links = ('name',)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == 'device':
            kwargs['queryset'] = Device.objects.filter(protocol=PROTOCOL_ID)
        return super(SerialVariableAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)

    def get_queryset(self, request):
        """Limit Pages to those that belong to the request's user."""
        qs = super(SerialVariableAdmin, self).get_queryset(request)
        return qs.filter(device__protocol_id=PROTOCOL_ID)

    inlines = [
        SerialVariableAdminInline
    ]


# admin_site.register(ExtendedSerialDevice, SerialDeviceAdmin)
# admin_site.register(ExtendedSerialVariable, SerialVariableAdmin)
